package wanijo.wanijo2.domain

import org.springframework.data.annotation.Id
import org.springframework.data.annotation.Transient
import org.springframework.data.jdbc.repository.query.Query
import org.springframework.data.relational.core.mapping.Table
import org.springframework.data.repository.Repository
import java.time.ZonedDateTime

@Table("T_DOCUMENT")
data class Document(
    @Id
    val id: Long = 0,
    val name: String,
    val description: String = "",
    val updatedAt: ZonedDateTime = ZonedDateTime.now(),
    val createdAt: ZonedDateTime = ZonedDateTime.now(),
    val labelFields: Set<LabelField> = emptySet(),
    val dateFields: Set<DateField> = emptySet(),
)

interface DocumentDao: Repository<Document, Int> {
    fun findAll(): List<Document>
    fun findById(id: Long): Document?
    fun findByName(name: String): List<Document>
    fun save(doc: Document): Document

    @Query(
        """
            SELECT doc.*
              FROM t_document doc
              JOIN t_document_tagging tagging
                ON tagging.document_id = doc.id
              JOIN t_tag tag
                ON tagging.tag_id = tag.id
             WHERE LOWER(tag.name) LIKE '%' || LOWER(:tagName) || '%'
        """
    )
    fun findByTagName(tagName: String): List<Document>
}
