package wanijo.wanijo2.domain

import org.springframework.data.annotation.Id
import org.springframework.data.jdbc.core.mapping.AggregateReference
import org.springframework.data.jdbc.repository.query.Query
import org.springframework.data.relational.core.mapping.Table
import org.springframework.data.repository.Repository
import java.time.ZonedDateTime

@Table("T_TAG")
data class Tag(
    @Id
    val id: Long = 0,
    val name: String,
    val createdAt: ZonedDateTime = ZonedDateTime.now()
)

@Table("T_DOCUMENT_TAGGING")
data class DocumentTagging(
    @Id
    val id: Long = 0,
    val tagId: AggregateReference<Tag, Long>,
    val documentId: AggregateReference<Document, Long>
) {
    companion object {
        fun between(doc: Document, tagId: Long) =
            DocumentTagging(
                tagId = AggregateReference.to(tagId),
                documentId = AggregateReference.to(doc.id)
            )
    }
}

interface DocumentTaggingDao : Repository<DocumentTagging, Long> {
    fun save(tagging: DocumentTagging)
}

interface TagDao : Repository<Tag, Long> {
    fun save(tag: Tag): Tag
    fun findByName(name: String): Tag?
    fun findAll(): Set<Tag>
    @Query(
        """
            SELECT tag.*
              FROM t_tag tag
              JOIN t_document_tagging tdt
                ON tag.id = tdt.tag_id
              JOIN t_document td
                ON tdt.document_id = td.id
             WHERE td.id = :documentId
        """
    )
    fun ofDocument(documentId: Long): Set<Tag>
}
