package wanijo.wanijo2.http.controller

import jakarta.validation.Valid
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.ui.set
import org.springframework.validation.BindingResult
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import wanijo.wanijo2.domain.TagDao
import wanijo.wanijo2.domain.event.NewDocumentCommand
import wanijo.wanijo2.domain.handler.NewDocumentCommandHandler

@Controller
@RequestMapping("/document")
class NewController(
    private val tagDao: TagDao,
    private val newDocumentCommandHandler: NewDocumentCommandHandler,
) {

    @GetMapping("/new")
    fun newDocument(
        model: Model
    ): String {
        model["tags"] = tagDao.findAll()
        model["form"] = NewDocumentCommand()
        return "new"
    }

    @PostMapping("/new")
    fun saveNewDocument(
        @Valid
        newDocument: NewDocumentCommand,
        model: Model,
        bindingResult: BindingResult
    ): String {
        if (bindingResult.hasErrors()) {
            return newDocument(model)
        }

        val doc = newDocumentCommandHandler.exec(newDocument)

        return "redirect:/document/${doc.id}"
    }

}
