package wanijo.wanijo2.domain

import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertNotNull
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.data.jdbc.core.mapping.AggregateReference
import org.springframework.test.annotation.DirtiesContext
import kotlin.test.assertEquals

@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_EACH_TEST_METHOD)
class DocumentDaoTest {

    @Autowired
    lateinit var adapter: DocumentDao

    @Autowired
    lateinit var tagDao: TagDao

    @Autowired
    lateinit var documentTaggingDao: DocumentTaggingDao

    @Test
    fun insertWorks() {
        assertEquals(0, adapter.findAll().count())

        adapter.save(
            Document(name = "name")
        )

        assertEquals(1, adapter.findAll().count())
    }

    @Test
    fun findByIdWorks() {
        val doc = Document(name = "name")

        adapter.save(doc)

        val savedDoc = adapter.findById(1)
        assertNotNull(savedDoc)
    }

    @Test
    fun idsAreGeneratedSequentially() {
        adapter.save(
            Document(name = "name1")
        )

        adapter.save(
            Document(name = "name2")
        )

        assertNotNull(adapter.findById(1))
        assertNotNull(adapter.findById(2))
    }

    @Test
    fun findByNameWorks() {
        adapter.save(
            Document(name = "findmich")
        )

        assertNotNull(adapter.findByName("findmich"))
    }

    @Test
    fun findsDocumentsWithTag() {
        val tag1 = tagDao.save(Tag(name = "tAg1"))
        val tag2 = tagDao.save(Tag(name = "taG2"))

        val doc1 = adapter.save(
            Document(
                name = "Dok1",
            )
        )

        val doc2 = adapter.save(
            Document(
                name = "Dok2",
            )
        )

        documentTaggingDao.save(
            DocumentTagging(
                tagId = AggregateReference.to(tag1.id),
                documentId = AggregateReference.to(doc1.id)
            )
        )

        documentTaggingDao.save(
            DocumentTagging(
                tagId = AggregateReference.to(tag2.id),
                documentId = AggregateReference.to(doc2.id)
            )
        )

        val docs = adapter.findByTagName("tag1")
        assertEquals(1, docs.size)
        assertEquals("Dok1", docs[0].name)
    }

}
